#############################
###                       ###
###     Jarbin-ToolKit    ###
###        console        ###
###   ----cursor.py----   ###
###                       ###
###=======================###
### by JARJARBIN's STUDIO ###
#############################


from builtins import object
from typing import Any
from jarbin_toolkit_console.System.setting import Setting


Setting.update()


class Cursor:
    """
        Cursor class.

        Manipulate the cursor's position.
    """

    from jarbin_toolkit_console.ANSI.ansi import ANSI


    @staticmethod
    def up(
            n: int = 1
        ) -> ANSI:
        """
            Bring the cursor up 'n' lines

            Parameters:
                n (int, optional): number of lines up

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}A")


    @staticmethod
    def down(
            n: int = 1
        ) -> ANSI:
        """
            Bring the cursor down 'n' lines

            Parameters:
                n (int, optional): number of lines down

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}B")


    @staticmethod
    def left(
            n: int = 1
        ) -> ANSI:
        """
            Bring the cursor left 'n' column

            Parameters:
                n (int, optional): number of column left

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}D")


    @staticmethod
    def right(
            n: int = 1
        ) -> ANSI:
        """
            Bring the cursor right 'n' column

            Parameters:
                n (int, optional): number of column right

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}C")


    @staticmethod
    def top(
        ) -> ANSI:
        """
            Move the cursor to the top left corner of the console

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}H")


    @staticmethod
    def previous(
            n: int = 1
        ) -> ANSI:
        """
            Move the cursor to the beginning of the 'n' previous line

            Parameters:
                n (int, optional): number of column up

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}F")


    @staticmethod
    def next(
            n: int = 1
        ) -> ANSI:
        """
            Move the cursor to the beginning of the 'n' next line

            Parameters:
                n (int, optional): number of column right

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{n}E")


    @staticmethod
    def move(
            y : int = 0,
            x : int = 0
        ) -> ANSI:
        """
            Move the cursor to the column x and line y

            Parameters:
                y (int, optional): row y position
                x (int, optional): column x position

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{y};{x}H")


    @staticmethod
    def move_column(
            x : int = 0
        ) -> ANSI:
        """
            Move the cursor to the column x and line y

            Parameters:
                x (int, optional): column x position

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}{x}G")


    @staticmethod
    def set(
        ) -> ANSI:
        """
            Save the cursor's position

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}7")


    @staticmethod
    def reset(
        ) -> ANSI:
        """
            Move the cursor to the saved position

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}8")


    @staticmethod
    def show(
        ) -> ANSI:
        """
            Show the cursor

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}?25h")


    @staticmethod
    def hide(
        ) -> ANSI:
        """
            Hide the cursor

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}?25l")



#############################
###                       ###
###     Jarbin-ToolKit    ###
###        console        ###
###     ----line.py----   ###
###                       ###
###=======================###
### by JARJARBIN's STUDIO ###
#############################


from builtins import object
from typing import Any
from jarbin_toolkit_console.System.setting import Setting


Setting.update()


class Line:
    """
        Line class.

        Manipulate the lines of the console.
    """

    from jarbin_toolkit_console.ANSI.ansi import ANSI


    @staticmethod
    def clear_line(
        ) -> ANSI:
        """
            Clear the current line

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}2K")


    @staticmethod
    def clear_start_line(
        ) -> ANSI:
        """
            Clear the current line from the start to the cursor's position

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}1K")


    @staticmethod
    def clear_end_line(
        ) -> ANSI:
        """
            Clear the current line from the cursor's position to the end

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}K")


    @staticmethod
    def clear_screen(
        ) -> ANSI:
        """
            Clear the screen

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.ansi import ANSI

        return ANSI(f"{ANSI.ESC}2J")


    @staticmethod
    def clear(
        ) -> ANSI:
        """
            Clear the screen and bring the cursor to the top left corner

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.cursor import Cursor

        return Line.clear_screen() + Cursor.top()


    @staticmethod
    def clear_previous_line(
            n : int = 1
    ) -> ANSI:
        """
            Clear the previous line.

            Parameters:
                n (int): line up

            Returns:
                ANSI: ansi sequence
        """

        from jarbin_toolkit_console.ANSI.cursor import Cursor

        return Cursor.previous(n) + Line.clear_line()



import pytest


from jarbin_toolkit_console.Animation import BasePack, Style
from jarbin_toolkit_console import (init, quit)


init()


def test_basepack_has_attributes(
    ) -> None:
    assert hasattr(BasePack, "P_SLIDE_R")
    assert hasattr(BasePack, "P_SLIDE_L")
    assert hasattr(BasePack, "P_SLIDER_R")
    assert hasattr(BasePack, "P_SLIDER_L")
    assert hasattr(BasePack, "P_FILL_R")
    assert hasattr(BasePack, "P_FILL_L")
    assert hasattr(BasePack, "P_EMPTY_R")
    assert hasattr(BasePack, "P_EMPTY_L")
    assert hasattr(BasePack, "P_FULL")
    assert hasattr(BasePack, "P_EMPTY")


def test_basepack_types(
    ) -> None:
    assert isinstance(BasePack.P_SLIDE_R, list)
    assert isinstance(BasePack.P_SLIDE_L, list)
    assert isinstance(BasePack.P_SLIDER_R, list)
    assert isinstance(BasePack.P_SLIDER_L, list)
    assert isinstance(BasePack.P_FILL_R, list)
    assert isinstance(BasePack.P_FILL_L, list)
    assert isinstance(BasePack.P_EMPTY_R, list)
    assert isinstance(BasePack.P_EMPTY_L, list)
    assert isinstance(BasePack.P_FULL, list)
    assert isinstance(BasePack.P_EMPTY, list)


def test_basepack_update_with_style(
    ) -> None:
    style = Style(on="X", off="_", arrow_left="{", arrow_right="}", border_left="[", border_right="]")
    BasePack.update(style)

    assert any("X" in frame or "_" in frame for frame in BasePack.P_SLIDE_R)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_SLIDE_L)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_SLIDER_R)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_SLIDER_L)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_FILL_R)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_FILL_L)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_EMPTY_R)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_EMPTY_L)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_FULL)
    assert any("X" in frame or "_" in frame for frame in BasePack.P_EMPTY)


def test_basepack_update_without_style(
    ) -> None:
    BasePack.update()

    assert any("#" in frame or "-" in frame for frame in BasePack.P_SLIDE_R)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_SLIDE_L)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_SLIDER_R)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_SLIDER_L)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_FILL_R)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_FILL_L)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_EMPTY_R)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_EMPTY_L)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_FULL)
    assert any("#" in frame or "-" in frame for frame in BasePack.P_EMPTY)


quit(delete_log=True)
